;Infrarotempfnger mit Tiny2313 fr Pollin-IR-Bausatz

;Vollversion fr Tipp-Betrieb und Toggle-Betrieb.

;Durch Auswechseln des Pollin-AVRs gegen einen Tiny2313 mit diesem Programm
;entsteht ein vllig neuer IR-Empfnger, der mit derselben Fernbedienung
;angesprochen werden kann.

;Damit Parallelbetrieb mit dem Pollin-Original mglich ist, nutzt dieser
;Empfnger andere, vom Pollin-Projekt nicht genutzte Tasten.

;Der Handsender kann neben dem Original-Empfnger von Pollin 16 Empfnger
;mit diesem Programm ansprechen, die durch Jumper an den Pins 2,3,4 und 5 des
;Tiny2313 unterschieden werden.

.include "tn2313def.inc"

.equ clock=1000000  ;1MHz Taktfrequenz
.equ signal=pd2     ;Signaleingang
.equ led1=pd3       ;Empfangs-LED-Ausgang
.equ led2=pd4       ;Scharf-LED-Ausgang
.equ shift0=0b1111111111100000  ;Bitmuster fr Empfangs-Start
.equ timout=50      ;Timeout fr Impulsabstand
.equ dauer0=8       ;Anzeigedauer etwa 0,5 Sekunden
.equ adr0=7         ;Adresse der Fernbedienung (7)
.equ bli0=5         ;halbe Blinkperiode etwa 1/3 Sekunde

.dseg               ;SRAM
.org $60            ;ab Adresse 196 (nach mapped-I/O-Bereich)
dauerz:     .byte 10    ;Ausschalt-Zhler fr die Relais

.def srsk=r2        ;SREG-Kopie
.def null=r3        ;immer 0
.def shiftl=r4      ;Empfangsregister L
.def shifth=r5      ;Empfangsregister H
.def zsalt=r6       ;alter Zeitstempel
.def zaehler=r7     ;freilaufender Software-Timer 256 s
.def poko=r8        ;Port-Kopie vom Eingang
.def tout=r9        ;Timeout Impulsabstand
.def cmdalt=r10     ;zuletzt empfangenes Kommando (fr Entprellung)
.def cmd=r11        ;Kommando-Wert nach RECS80-Norm
.def gnr=r12        ;Gertenummer laut Jumper
.def adr=r13        ;Adresse (momentan aktiviertes Gert)
.def bliz=r14       ;Blinkzhler

.def flags=r16      ;einige Flags
    .equ zovf=7         ;Zhler-berlauf fr Zeitstempel
    .equ iovf=6         ;Impulsbreite-berlauf
    .equ neu=5          ;neues Telegramm eingetroffen
    .equ nichtaus=4     ;Relais nicht ausschalten

.def tas=r17        ;Tasten-Wert nach eigener Norm
.def dauer=r18      ;Dauer des Schaltvorgangs am Port
.def wl=r24         ;Working L
.def wh=r25         ;Working H

.cseg               ;Programmspeicher
.org 0              ;ab Adresse 0
 rjmp RESET         ;Reset Handler
 rjmp exint0        ;External Interrupt0 Handler
 rjmp nix;INT1          ;External Interrupt1 Handler
 rjmp nix;TIM1_CAPT     ;Timer1 Capture Handler
 rjmp nix;TIM1_COMPA    ;Timer1 CompareA Handler
 rjmp nix;TIM1_OVF      ;Timer1 Overflow Handler
 rjmp TIM0_OVF      ;Timer0 Overflow Handler
 rjmp nix;USART0_RXC    ;USART0 RX Complete Handler
 rjmp nix;USART0_DRE    ;USART0,UDR Empty Handler
 rjmp nix;USART0_TXC    ;USART0 TX Complete Handler
 rjmp nix;ANA_COMP      ;Analog Comparator Handler
 rjmp nix;PCINT         ;Pin Change Interrupt
 rjmp nix;TIMER1_COMPB  ;Timer1 Compare B Handler
 rjmp nix;TIMER0_COMPA  ;Timer0 Compare A Handler
 rjmp nix;TIMER0_COMPB  ;Timer0 Compare B Handler
 rjmp nix;USI_START     ;USI Start Handler
 rjmp nix;USI_OVERFLOW  ;USI Overflow Handler
 rjmp nix;EE_READY      ;EEPROM Ready Handler
 rjmp nix;WDT_OVERFLOW  ;Watchdog Overflow Handler
nix:                    ;unerlaubte Interrupts
 rjmp nix                   ;Falle...
 reti                       ;zurck...

reset:                  ;Initialisierung
 ldi wl,low(ramend)         ;Stackpointer
 out spl,wl                 ;initialisieren
 clr null                   ;immer 0
 ldi wl,$ff                 ;alle Pins von
 out ddrb,wl                ;PortB als Ausgang
 out portb,null             ;und L-Pegel
 ldi wl,0b10000111          ;PullUps fr Eingnge
 out portd,wl               ;einschalten (auer LED)
 out porta,wl               ;auch PortA
 com wl                     ;LEDs und Relais 9 und 10 als Ausgang
 out ddrd,wl                ;Jumper und IR-Sensor als Eingang
 out ddra,null              ;PortA (Jumper) als Eingang
 
 out tccr0a,null            ;keine Sonderfunktionen
 ldi wl,1                   ;Vorteiler 1:1
 out tccr0b,wl              ;fr Timer0
 ldi wl,1<<toie0            ;OVF0-Interrupt
 out timsk,wl               ;einschalten
 ldi wl,(1<<se)|(1<<isc00)  ;Sleep-Mode Idle und wechselnde Flanke
 out mcucr,wl               ;fr Int0 vorwhlen
 ldi wl,1<<int0             ;Int0
 out gimsk,wl               ;aktivieren
 ldi wl,15                  ;erstmal Adresse 15, also alle Jumper offen
 mov adr,wl                 ;
 sei                        ;Interrupts freigeben
 clr tas                    ;ungltige Taste
 in poko,pind               ;Jumper einlesen
 rcall neutelda0            ;Adresse testen...

mainloop:               ;Hauptschleife
 sbrc flags,zovf            ;Port nach gewisser Zeit
 rcall machaus              ;abschalten
 sbrc flags,neu             ;neues Telegramm eingetroffen? - nein
 rcall neutelda             ;ja, behandeln...
 sleep                      ;bis zum nchsten Interrupt pennen
 rjmp mainloop              ;nochmal...

machaus:                ;Relais wieder ausschalten
 cbr flags,1<<zovf          ;Jobflag lschen
 ldi yl,low(dauerz)         ;Y-Pointer auf Adresse des
 ldi yh,high(dauerz)        ;ersten Ausschalt-Timers
 ldi zl,low(reltab*2)       ;Pointer auf
 ldi zh,high(reltab*2)      ;Relais-Bitmuster
machaus00:
 lpm r0,z+                  ;Bitmuster fr PortB
 lpm r1,z+                  ;und PortD holen
 com r0                     ;Bitmuster
 com r1                     ;invertieren
 ld wl,y                    ;Zhlerstand holen (Pointer nicht verndern)
 tst wl                     ;wenn schon 0,
 breq machaus01             ;dann fertig...
 dec wl                     ;Timeout Relais runter
 brne machaus01             ;schon unten? - nein...
 in wh,portb                ;PortB einlesen
 and wh,r0                  ;auszuschaltendes Bit lschen
 out portb,wh               ;zurckschreiben
 in wh,portd                ;PortD einlesen
 and wh,r1                  ;auszuschaltendes Bit lschen
 out portd,wh               ;zurckschreiben
machaus01:
 st y+,wl                   ;Timer zurckschreiben und Pointer erhhen
 cpi yl,dauerz+10           ;alle 10 Zhler behandelt?
 brne machaus00             ;nein, nochmal...
                            ;ja
 cp gnr,adr                 ;Gert scharf?
 brne machaus4              ;nein...
 inc bliz                   ;ja, Blinkzhler erhhen
 sbrs flags,nichtaus        ;Dauerstrom-Mode? - ja...
 rjmp machaus3              ;nein...
 mov wl,bliz                ;Kopie wegen Konstanten
 cpi wl,bli0                ;Endwert erreicht?
 brcs machaus2              ;nein...
 clr bliz                   ;ja, von vorn und
 sbi pind,led2              ;LED toggeln
machaus2:
 ret
machaus3:
 sbi portd,led2
 ret
machaus4:
 cbi portd,led2
 ret

neutelda:               ;neues Telegramm eingetroffen
 cbr flags,1<<neu           ;Jobflag lschen
 mov zl,cmd                 ;Kommando-Nummer in Pointer
 clr zh                     ;kopieren
 andi zl,63                 ;Toggelbit ausblenden
 subi zl,low(-tastab*2)     ;Basisadresse der Tabelle
 sbci zh,high(-tastab*2)    ;addieren
 lpm tas,z                  ;Tastennummer holen
neutelda0:
 mov wl,poko                ;Port-Zustand
 andi wl,3                  ;nur untere 2 Bits
 in wh,pina                 ;Port A auch einlesen
 bst wh,1                   ;Jumper-Bits
 bld wl,2                   ;kopieren
 bst wh,0
 bld wl,3
 mov gnr,wl                 ;Gertenummer laut Jumper definieren
 sbrs tas,5                 ;Adresstaste? - ja...
 rjmp neutelda1             ;nein...
 andi tas,15                ;Wert maskieren
 mov adr,tas                ;und bernehmen
 rjmp neutelda3             ;fertig...
neutelda1: 
 cp adr,gnr                 ;ist dieses Gert momentan aktiv?
 brne neutelda3             ;nein...
 sbrs tas,4                 ;Relaistaste? - ja...
 rjmp neutelda3             ;nein...
 cpi tas,26                 ;Mode-Befehl?
 brne neutelda11            ;nein...
 ldi wl,1<<nichtaus         ;ja, anmelden
 eor flags,wl               ;oder abmelden
 rjmp neuteldaend           ;fertig...
neutelda11:
 ldi zl,low(reltab*2)       ;Pointer auf
 ldi zh,high(reltab*2)      ;Relais-Bitmuster
 andi tas,15                ;nur Tastenwert
 add zl,tas                 ;als Offset
 adc zh,null                ;mit bertrag addieren
 add zl,tas                 ;nochmal, da 2 Bytes pro Eintrag
 adc zh,null                ;mit bertrag addieren
 lpm wl,z+                  ;Bitmuster fr PortB holen,
 in wh,portb                ;Kopie von Zustand PortB
 eor wh,wl                  ;Bit in Portkopie toggeln
 out portb,wh               ;und wieder ausgeben
 lpm wl,z+                  ;Bitmuster fr PortD holen,
 in wh,portd                ;Kopie von Zustand PortD
 eor wh,wl                  ;Bit in Portkopie toggeln
 out portd,wh               ;und wieder ausgeben
neutelda2:
 sbrc flags,nichtaus        ;Dauerbetrieb? - nein...
 rjmp neutelda3             ;ja...
 ldi yl,low(dauerz)         ;Y-Pointer auf Adresse des
 ldi yh,high(dauerz)        ;ersten Ausschalt-Timers
 add yl,tas                 ;Relaisnummer dazu
 ;adc yh,null                ;bertrag auch
 ldi wl,dauer0              ;Zhler fr Anzeigedauer
 st y,wl                    ;setzen
neutelda3:
 clr tas                    ;Tastennummer lschen
 cbi portd,led2             ;Scharf-LED erstmal ausschalten
 cp gnr,adr                 ;Gert scharf?
 brne neuteldaend           ;nein...
 sbi portd,led2             ;ja, Scharf-LED einschalten
neuteldaend:
 clr tas                    ;Tastenwert lschen
 ret                        ;fertig

TIM0_OVF:               ;ISR alle 256 Takte (256 s)
 in srsk,sreg               ;SREG sichern
 inc zaehler                ;Softwaretimer erhhen
 brne tim0_ovf1             ;berlauf? - nein...
 sbr flags,1<<zovf          ;ja, dem Hauptprogramm mitteilen
tim0_ovf1:
 cp zaehler,tout            ;Timeout Impulsabstand erreicht?
 brne isrend                ;nein...
imperr:                 ;Impulserror, neues Telegramm vorbereiten
 mov zsalt,zaehler          ;Zeitstempel holen
 ldi xl,timout              ;Timeout
 add xl,zsalt               ;berechnen und
 out ocr0a,xl               ;setzen
 ldi xl,low(shift0)         ;Start-
 mov shiftl,xl              ;Bitmuster
 ldi xl,high(shift0)        ;ins Shiftregister
 mov shifth,xl              ;schreiben
isrend:
 out sreg,srsk              ;SREG wiederherstellen
 reti                       ;fertig...
 
exint0:                 ;ISR Impuls vom Empfnger
 in srsk,sreg               ;SREG sichern
 in poko,pind               ;Kopie vom Eingangsport zur Strunterdrckung
 sbrc poko,signal           ;fallende Flanke? - ja...
 rjmp exint01               ;nein, steigende Flanke...
 sbi portd,led1             ;RX-LED ein
 rjmp isrend                ;fertig
exint01:
 cbi portd,led1             ;ja, RX-LED aus
 mov xl,zaehler             ;ja, Impuls gltig, aktuellen Zeitstempel holen
 sub xl,zsalt               ;Differenz zum vorigen Zeitstempel
 mov zsalt,zaehler          ;neuen Zeitstempel merken
 ldi wh,timout              ;Timeout fr Impulsabstanderror 
 mov tout,zaehler           ;auf "jetzt+x" 
 add tout,wh                ;setzen
 cpi xl,17                  ;Impulsabstand unter 0-Wert? (18..22)
 brcs imperr                ;ja, unter 18, Fehler...
 cpi xl,23                  ;Impulsabstand noch 0-Wert? (18..22)
 brcc exint02               ;nein, ber 22, kein 0-Bit mehr
 clc                        ;ja, 0-Bit, Carry lschen
 rjmp shiftin               ;und einschieben
exint02:
 cpi xl,27                  ;Impulsabstand unter 1-Wert? (28..32)
 brcs imperr                ;ja, unter 28, Fehler...
 cpi xl,33                  ;Impulsabstand noch 1-Wert? (28..32)
 brcc imperr                ;nein, ber 32, kein 1-Bit mehr, Error
 sec                        ;ja, 1-Bit, Carry setzen

shiftin:                ;schiebt Carry ein
 rol shiftl                 ;Carry ins untere Shiftregister schieben
 rol shifth                 ;berlauf ins obere Shiftregister schieben
 sbrc shifth,7              ;komplettes Telegramm eingelesen? - ja...
 rjmp isrend                ;nein, fertig...
;Telegramm ist komplett eingelesen
 lsl shiftl                 ;Adressbits
 rol shifth                 ;nach
 lsl shiftl                 ;oben
 rol shifth                 ;schieben,
 lsr shiftl                 ;Command wieder
 lsr shiftl                 ;zurck
 bst shifth,3               ;Togglebit in
 bld shiftl,6               ;Command kopieren
 ldi xl,7                   ;nur die 
 and shifth,xl              ;Adressbits maskieren
 ldi xl,adr0                ;zum Vergleich
 cp shifth,xl               ;stimmt die Adresse?
 breq shiftin1              ;ja...
 rjmp imperr                ;nein, verwerfen...
shiftin1:               ;Adresse stimmt
 cp shiftl,cmd              ;dieses Telegramm schon erkannt?
 breq shiftin1a             ;ja, verwerfen...
 cp shiftl,cmdalt           ;empfangenes Telegramm stabil?
 breq shiftin2              ;ja...
 mov cmdalt,shiftl          ;nein, neues Telegramm merken,
shiftin1a:
 rjmp imperr                ;neuen Empfang vorbereiten...
shiftin2:               ;Telegramm stimmt mit vorherigem berein
 mov cmd,shiftl             ;Kommandowert bernehmen
 sbr flags,1<<neu           ;neues Telegramm melden
 rjmp imperr                ;nchstes Telegramm vorbereiten...


tastab: ;Tabelle mit selbst definierten Tastennummern (64 Eintrge)
.db  0, 0, 0, 0,36,37,33, 0 ;00..07
.db  0, 0, 0, 0, 0, 0, 0,38 ;08..15
.db 46, 0, 0, 0, 0, 0, 0, 0 ;16..23
.db  0,45,40,41,44,32,43,47 ;24..31
.db 20,21,24,25,18,19,16,17 ;32..39
.db 22,23,22,23, 0, 0, 0, 0 ;40..47
.db  0,26, 0, 0,42, 0, 0, 0 ;48..55
.db 39, 0, 0, 0, 0, 0,34,35 ;56..63

reltab: ;Tabelle mit Bitmustern fr die Relais (10 Eintrge)
.db   1,  0     ;Bitmuster fr PortB und PortD
.db   2,  0
.db   4,  0
.db   8,  0
.db  16,  0
.db  32,  0
.db  64,  0
.db 128,  0
.db   0, 32
.db   0, 64





