;Infrarotempfnger mit Tiny2313 fr Pollin-IR-Bausatz

;Spezialversion fr Fabian Wolf zum Bedienen eines Eigenbau-Radios.

;Durch Auswechseln des Pollin-AVRs gegen einen Tiny2313 mit diesem Programm
;entsteht ein vllig neuer IR-Empfnger, der mit derselben Fernbedienung
;angesprochen werden kann.


.include "tn2313def.inc"

.equ clock=1000000  ;1MHz Taktfrequenz
.equ signal=pd2     ;Signaleingang
.equ led1=pd3       ;Empfangs-LED-Ausgang
.equ shift0=0b1111111111100000  ;Bitmuster fr Empfangs-Start
.equ timout=50      ;Timeout fr Impulsabstand
.equ adr0=7         ;Adresse der Fernbedienung (7)

.dseg               ;SRAM
.org $60            ;ab Adresse 196 (nach mapped-I/O-Bereich)
dauerz:     .byte 8 ;Ausschalt-Zhler fr die Relais

.def srsk=r2        ;SREG-Kopie
.def null=r3        ;immer 0
.def shiftl=r4      ;Empfangsregister L
.def shifth=r5      ;Empfangsregister H
.def zsalt=r6       ;alter Zeitstempel
.def zaehler=r7     ;freilaufender Software-Timer 256 s
.def poko=r8        ;Port-Kopie vom Eingang
.def tout=r9        ;Timeout Impulsabstand
.def cmdalt=r10     ;zuletzt empfangenes Kommando (fr Entprellung)
.def cmd=r11        ;Kommando-Wert nach RECS80-Norm
.def afcz=r12       ;AFC-Zhler (Feinabstimmung)

.def flags=r16      ;einige Flags
    .equ zovf=7         ;Zhler-berlauf fr Zeitstempel
    .equ iovf=6         ;Impulsbreite-berlauf
    .equ neu=5          ;neues Telegramm eingetroffen

.def tas=r17        ;Tasten-Wert nach eigener Norm
.def dauer=r18      ;Dauer des Schaltvorgangs am Port
.def wl=r24         ;Working L
.def wh=r25         ;Working H

.cseg               ;Programmspeicher
.org 0              ;ab Adresse 0
 rjmp RESET         ;Reset Handler
 rjmp exint0        ;External Interrupt0 Handler
 rjmp nix;INT1          ;External Interrupt1 Handler
 rjmp nix;TIM1_CAPT     ;Timer1 Capture Handler
 rjmp nix;TIM1_COMPA    ;Timer1 CompareA Handler
 rjmp nix;TIM1_OVF      ;Timer1 Overflow Handler
 rjmp TIM0_OVF      ;Timer0 Overflow Handler
 rjmp nix;USART0_RXC    ;USART0 RX Complete Handler
 rjmp nix;USART0_DRE    ;USART0,UDR Empty Handler
 rjmp nix;USART0_TXC    ;USART0 TX Complete Handler
 rjmp nix;ANA_COMP      ;Analog Comparator Handler
 rjmp nix;PCINT         ;Pin Change Interrupt
 rjmp nix;TIMER1_COMPB  ;Timer1 Compare B Handler
 rjmp nix;TIMER0_COMPA  ;Timer0 Compare A Handler
 rjmp nix;TIMER0_COMPB  ;Timer0 Compare B Handler
 rjmp nix;USI_START     ;USI Start Handler
 rjmp nix;USI_OVERFLOW  ;USI Overflow Handler
 rjmp nix;EE_READY      ;EEPROM Ready Handler
 rjmp nix;WDT_OVERFLOW  ;Watchdog Overflow Handler
nix:                    ;unerlaubte Interrupts
 rjmp nix                   ;Falle...
 reti                       ;zurck...

reset:                  ;Initialisierung
 ldi wl,low(ramend)         ;Stackpointer
 out spl,wl                 ;initialisieren
 clr null                   ;immer 0
 clr afcz                   ;Zhler Feinabstimmung auf 0
 ldi wl,$ff                 ;alle Pins von
 out ddrb,wl                ;PortB als Ausgang
 out portb,null             ;und L-Pegel
 ldi wl,0b10000111          ;PullUps fr Eingnge
 out portd,wl               ;einschalten (auer LED)
 out porta,wl               ;auch PortA
 com wl                     ;LEDs und Relais 9 und 10 als Ausgang
 out ddrd,wl                ;Jumper und IR-Sensor als Eingang
 out ddra,null              ;PortA (Jumper) als Eingang
 
 out tccr0a,null            ;keine Sonderfunktionen
 ldi wl,1                   ;Vorteiler 1:1
 out tccr0b,wl              ;fr Timer0
 ldi wl,1<<toie0            ;OVF0-Interrupt
 out timsk,wl               ;einschalten
 ldi wl,(1<<se)|(1<<isc00)  ;Sleep-Mode Idle und wechselnde Flanke
 out mcucr,wl               ;fr Int0 vorwhlen
 ldi wl,1<<int0             ;Int0
 out gimsk,wl               ;aktivieren
 sei                        ;Interrupts freigeben
 clr tas                    ;ungltige Taste

mainloop:               ;Hauptschleife
 sbrc flags,zovf            ;Port nach gewisser Zeit
 rcall machaus              ;abschalten
 sbrc flags,neu             ;neues Telegramm eingetroffen? - nein
 rcall neutelda             ;ja, behandeln...
 sleep                      ;bis zum nchsten Interrupt pennen
 rjmp mainloop              ;nochmal...

machaus:                ;Relais wieder ausschalten
 cbr flags,1<<zovf          ;Jobflag lschen
 ldi yl,low(dauerz)         ;Y-Pointer auf Adresse des
 ldi yh,high(dauerz)        ;ersten Ausschalt-Timers
 ldi zl,low(reltab*2)       ;Pointer auf
 ldi zh,high(reltab*2)      ;Relais-Bitmuster
machaus00:
 lpm r0,z+                  ;Bitmuster fr PortB
 lpm r1,z+                  ;und PortD holen
 com r0                     ;Bitmuster
 com r1                     ;invertieren
 ld wl,y                    ;Zhlerstand holen (Pointer nicht verndern)
 tst wl                     ;wenn schon 0,
 breq machaus01             ;dann fertig...
 dec wl                     ;Timeout Relais runter
 brne machaus01             ;schon unten? - nein...
 in wh,portb                ;PortB einlesen
 and wh,r0                  ;auszuschaltendes Bit lschen
 out portb,wh               ;zurckschreiben
machaus01:
 st y+,wl                   ;Timer zurckschreiben und Pointer erhhen
 cpi yl,dauerz+8            ;alle 8 Zhler behandelt?
 brne machaus00             ;nein, nochmal...
 ret                        ;ja, fertig...


neutelda:               ;neues Telegramm eingetroffen
 cbr flags,1<<neu           ;Jobflag lschen
 mov tas,cmd                ;Kopie der Kommando-Nummer
 andi tas,63                ;Toggelbit ausblenden
; in 'tas' steht jetzt der Tastencode ohne Toggelbit
taste50:                ;fragt Standby-Taste 50 ab
 cpi tas,50                 ;Taste 50 gedrckt?
 brne taste49               ;nein, weiter...
 sbi pinb,0                 ;PB0 toggeln (umschalten)
 nop                        ;auf Portreaktion
 nop                        ;warten
 sbic portb,0               ;ist jetzt ausgeschaltet worden? - ja...
 rjmp neuteldaend           ;nein... 
 clr afcz                   ;ja, AFC-Zhler fr nchsten Start lschen
 out portb,null             ;alle Portpins lschen (Stromverbrauch)
 
taste49:                ;fragt Muting-Taste 49 ab
 sbis portb,0               ;ist das Gert berhaupt eingeschaltet? - ja...
 rjmp neuteldaend           ;nein, andere Tasten verwerfen...
 cpi tas,49                 ;Taste 49 gedrckt?
 brne taste43               ;nein, weiter...
 sbi pinb,7                 ;PB7 toggeln (umschalten)
taste43:                ;fragt SEnderwechsel-Taste 43 ab
 cpi tas,43                 ;Taste 43 gedrckt?
 brne taste42               ;nein, weiter...
 cbi portb,1                ;PB1 ausschalten
 sbi portb,2                ;PB2 einschalten
 ldi wl,10                  ;10 Schritte a 0,5ms = 5ms
 sts dauerz+2,wl            ;in Abschaltzhler fr PB2 laden
 rjmp afc_aus               ;zur Feinabstimmung springen...
taste42:                ;fragt Senderwechsel-Taste 42 ab
 cpi tas,42                 ;Taste 42 gedrckt?
 brne taste33               ;nein, weiter...
 cbi portb,2                ;PB2 ausschalten
 sbi portb,1                ;PB1 einschalten
 ldi wl,10                  ;10 Schritte a 0,5ms = 5ms
 sts dauerz+1,wl            ;in Abschaltzhler fr PB1 laden
 rjmp afc_aus               ;zur Feinabstimmung springen...
taste33:                ;fragt Taste 33 ab
 cpi tas,33                 ;Taste 33 gedrckt?
 brne taste32               ;nein, weiter...
 cbi portb,1                ;PB1 ausschalten
 sbi pinb,2                 ;PB2 umschalten
 rjmp afc_ein               ;zur Feinabstimmung springen...
taste32:                ;fragt Taste 32 ab
 cpi tas,32                 ;Taste 32 gedrckt?
 brne taste35               ;nein, weiter...
 cbi portb,2                ;PB2 ausschalten
 sbi pinb,1                 ;PB1 umschalten
 rjmp afc_ein               ;zur Feinabstimmung springen...
taste35:                ;fragt Taste 35 ab
 cpi tas,35                 ;Taste 35 gedrckt?
 brne taste34               ;nein, weiter...
 cbi portb,5                ;PB5 ausschalten
 sbi portb,4                ;PB4 einschalten
taste34:                ;fragt Taste 34 ab
 cpi tas,34                 ;Taste 34 gedrckt?
 brne taste51               ;nein, weiter...
 cbi portb,4                ;PB4 ausschalten
 sbi portb,5                ;PB5 einschalten
taste51:                ;fragt AFC-aus-Taste 51 ab
 cpi tas,51                 ;Taste 51 gedrckt?
 brne neuteldaend           ;nein, alle Tasten durch, weiter...
afc_aus:                ;Feinabstimmung (AFC) ausschalten
 sbrs afcz,0                ;AFC-Zhlerstand ungerade? (AFC ein) - ja...
 rjmp neuteldaend           ;nein, nix tun...
afc:                    ;erzeugt AFC-Impuls an PB6
 sbi pinb,6                 ;PB6 einschalten,
 ldi wl,5                   ;5 Schritte a 1ms = 5ms
 sts dauerz+6,wl            ;in Abschaltzhler fr PB6 laden
 inc afcz                   ;AFC-Zhler erhhen
 sbrs afcz,0                ;AFC-Zustand
 cbi portb,3                ;an PB3 (LED?)
 sbrc afcz,0                ;anzeigen
 sbi portb,3                ;
neuteldaend:            ;Ende der Tastenauswertung
 clr tas                    ;Tastenwert lschen
 ret                        ;fertig

afc_ein:                ;Feinabstimmung (AFC) einschalten
 sbrs afcz,0                ;AFC-Zhlerstand ungerade? (AFC ein) - ja...
 rjmp afc                   ;nein, AFC ist aus, einschalten...
 rjmp neuteldaend           ;ja, nix tun...
 

TIM0_OVF:               ;ISR alle 256 Takte (256 s)
 in srsk,sreg               ;SREG sichern
 inc zaehler                ;Softwaretimer erhhen
 mov xl,zaehler             ;Kopie zum Maskieren
 andi xl,3                  ;jedes 4. mal (1ms)
 brne tim0_ovf1             ;jetzt nicht
 sbr flags,1<<zovf          ;dem Hauptprogramm Zeittakt mitteilen
tim0_ovf1:
 cp zaehler,tout            ;Timeout Impulsabstand erreicht?
 brne isrend                ;nein...
imperr:                 ;Impulserror, neues Telegramm vorbereiten
 mov zsalt,zaehler          ;Zeitstempel holen
 ldi xl,timout              ;Timeout
 add xl,zsalt               ;berechnen und
 out ocr0a,xl               ;setzen
 ldi xl,low(shift0)         ;Start-
 mov shiftl,xl              ;Bitmuster
 ldi xl,high(shift0)        ;ins Shiftregister
 mov shifth,xl              ;schreiben
isrend:
 out sreg,srsk              ;SREG wiederherstellen
 reti                       ;fertig...
 
exint0:                 ;ISR Impuls vom Empfnger
 in srsk,sreg               ;SREG sichern
 in poko,pind               ;Kopie vom Eingangsport zur Strunterdrckung
 sbrc poko,signal           ;fallende Flanke? - ja...
 rjmp exint01               ;nein, steigende Flanke...
 sbi portd,led1             ;RX-LED ein
 rjmp isrend                ;fertig
exint01:
 cbi portd,led1             ;ja, RX-LED aus
 mov xl,zaehler             ;ja, Impuls gltig, aktuellen Zeitstempel holen
 sub xl,zsalt               ;Differenz zum vorigen Zeitstempel
 mov zsalt,zaehler          ;neuen Zeitstempel merken
 ldi wh,timout              ;Timeout fr Impulsabstanderror 
 mov tout,zaehler           ;auf "jetzt+x" 
 add tout,wh                ;setzen
 cpi xl,17                  ;Impulsabstand unter 0-Wert? (18..22)
 brcs imperr                ;ja, unter 18, Fehler...
 cpi xl,23                  ;Impulsabstand noch 0-Wert? (18..22)
 brcc exint02               ;nein, ber 22, kein 0-Bit mehr
 clc                        ;ja, 0-Bit, Carry lschen
 rjmp shiftin               ;und einschieben
exint02:
 cpi xl,27                  ;Impulsabstand unter 1-Wert? (28..32)
 brcs imperr                ;ja, unter 28, Fehler...
 cpi xl,33                  ;Impulsabstand noch 1-Wert? (28..32)
 brcc imperr                ;nein, ber 32, kein 1-Bit mehr, Error
 sec                        ;ja, 1-Bit, Carry setzen

shiftin:                ;schiebt Carry ein
 rol shiftl                 ;Carry ins untere Shiftregister schieben
 rol shifth                 ;berlauf ins obere Shiftregister schieben
 sbrc shifth,7              ;komplettes Telegramm eingelesen? - ja...
 rjmp isrend                ;nein, fertig...
;Telegramm ist komplett eingelesen
 lsl shiftl                 ;Adressbits
 rol shifth                 ;nach
 lsl shiftl                 ;oben
 rol shifth                 ;schieben,
 lsr shiftl                 ;Command wieder
 lsr shiftl                 ;zurck
 bst shifth,3               ;Togglebit in
 bld shiftl,6               ;Command kopieren
 ldi xl,7                   ;nur die 
 and shifth,xl              ;Adressbits maskieren
 ldi xl,adr0                ;zum Vergleich
 cp shifth,xl               ;stimmt die Adresse?
 breq shiftin1              ;ja...
 rjmp imperr                ;nein, verwerfen...
shiftin1:               ;Adresse stimmt
 cp shiftl,cmd              ;dieses Telegramm schon erkannt?
 breq shiftin1a             ;ja, verwerfen...
 cp shiftl,cmdalt           ;empfangenes Telegramm stabil?
 breq shiftin2              ;ja...
 mov cmdalt,shiftl          ;nein, neues Telegramm merken,
 rjmp imperr                ;neuen Empfang vorbereiten...
shiftin1a:
 ldi xl,255                 ;Timeout direkt gesteuerter Relais
 sts dauerz+4,xl            ;Ausschalten von PB4 und PB5
 sts dauerz+5,xl            ;anmelden bzw. verzgern
 rjmp imperr                ;neuen Empfang vorbereiten...
shiftin2:               ;Telegramm stimmt mit vorherigem berein
 mov cmd,shiftl             ;Kommandowert bernehmen
 sbr flags,1<<neu           ;neues Telegramm melden
 rjmp imperr                ;nchstes Telegramm vorbereiten...


reltab: ;Tabelle mit Bitmustern fr die Relais (10 Eintrge)
.db   1,  0     ;Bitmuster fr PortB und PortD
.db   2,  0
.db   4,  0
.db   8,  0
.db  16,  0
.db  32,  0
.db  64,  0
.db 128,  0
.db   0, 32
.db   0, 64

